@@ -26,6 +26,10 @@ from sushy import utils
 LOG = logging.getLogger(__name__)
 
 
+_SERVER_SIDE_RETRIES = 5
+_SERVER_SIDE_RETRY_DELAY = 3
+
+
 class Connector(object):
 
     def __init__(
@@ -36,6 +40,7 @@ class Connector(object):
         self._session = requests.Session()
         self._session.verify = self._verify
         self._response_callback = response_callback
+        self._auth = None
 
         # NOTE(etingof): field studies reveal that some BMCs choke at
         # long-running persistent HTTP connections (or TCP connections).
@@ -68,7 +73,8 @@ class Connector(object):
         self._session.close()
 
     def _op(self, method, path='', data=None, headers=None, blocking=False,
-            timeout=60, **extra_session_req_kwargs):
+            timeout=60, server_side_retries=_SERVER_SIDE_RETRIES,
+            **extra_session_req_kwargs):
         """Generic RESTful request handler.
 
         :param method: The HTTP method to be used, e.g: GET, POST,
@@ -88,7 +94,10 @@ class Connector(object):
         url = path if urlparse.urlparse(path).netloc else urlparse.urljoin(
             self._url, path)
         headers = headers or {}
-        if not any(k.lower() == 'odata-version' for k in headers):
+        lc_headers = [k.lower() for k in headers]
+        if data is not None and 'content-type' not in lc_headers:
+            headers['Content-Type'] = 'application/json'
+        if 'odata-version' not in lc_headers:
             headers['OData-Version'] = '4.0'
         # TODO(lucasagomes): We should mask the data to remove sensitive
         # information
@@ -117,7 +126,7 @@ class Connector(object):
         try:
             exceptions.raise_for_response(method, url, response)
         except exceptions.AccessError as e:
-            if self._auth.can_refresh_session():
+            if self._auth is not None and self._auth.can_refresh_session():
                 try:
                     self._auth.refresh_session()
                 except exceptions.AccessError as refresh_exc:
@@ -137,9 +146,26 @@ class Connector(object):
                               "%s", retry_exc.message)
                     raise
             else:
-                LOG.error("Authentication error detected. Cannot proceed: "
-                          "%s", e.message)
+                if method == 'GET' and url.endswith('SessionService'):
+                    LOG.debug('HTTP GET of SessionService failed %s, '
+                              'this is expected prior to authentication',
+                              e.message)
+                else:
+                    LOG.error("Authentication error detected. Cannot proceed: "
+                              "%s", e.message)
                 raise
+        except exceptions.ServerSideError as e:
+            if method.lower() != 'get' or server_side_retries <= 0:
+                raise
+            else:
+                LOG.warning('Got server side error %s in response to a '
+                            'GET request, retrying after %d seconds',
+                            e, _SERVER_SIDE_RETRY_DELAY)
+                time.sleep(_SERVER_SIDE_RETRY_DELAY)
+                return self._op(method, path, data=data, headers=headers,
+                                blocking=blocking, timeout=timeout,
+                                server_side_retries=server_side_retries - 1,
+                                **extra_session_req_kwargs)
 
         if blocking and response.status_code == 202:
             if not response.headers.get('location'):
@@ -163,6 +189,7 @@ class Connector(object):
                             'timeout': timeout})
                     raise exceptions.ConnectionError(url=url, error=m)
             response = mon.response
+            exceptions.raise_for_response(method, url, response)
 
         LOG.debug('HTTP response for %(method)s %(url)s: '
                   'status code: %(code)s',
